﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;

namespace GenericLibrary
{

    public class CatalogList<TId, TElement> : IEnumerable<TElement> where TElement : IIdentifiable<TId>
    {
        private readonly IList<TElement> _elements = new List<TElement>();

        public CatalogList() { }

        public CatalogList(IEnumerable<TElement> otherCatalog)
        {
            foreach (TElement element in otherCatalog)
            {
                _elements.Add(element);
            }
        }

        public bool Add(TElement element)
        {
            if (_elements.Contains(element)) return false;
            _elements.Add(element);
            return true;
        }

        public int Count { get { return _elements.Count; } }

        public TElement Find(TId id)
        {
            foreach (TElement element in _elements)
            {
                if (element.Id.Equals(id))
                {
                    return element;
                }
            }
            return default(TElement);
        }

        public ICollection<TElement> GetAll()
        {
            return _elements;
        }

        public IReadOnlyCollection<TElement> GetAllReadOnlyCollection()
        {
            return new ReadOnlyCollection<TElement>(_elements);
        }

        public ICollection<TElement> GetAllAsReadOnly()
        {
            return _elements.ToList().AsReadOnly();
        }

        public ICollection<TElement> GetAllCopy()
        {
            //return new Collection<TElement>(_elements); // thin wrapper
            ICollection<TElement> result = new Collection<TElement>();
            foreach (var element in _elements)
            {
                result.Add(element);
            }
            return result;
        }

        public IEnumerable<TElement> GetAllEnumerable()
        {
            return _elements;
        } 

       public IEnumerator<TElement> GetEnumerator()
        {
            return _elements.GetEnumerator();
        }

        public override string ToString()
        {
            return String.Join(", ", _elements);
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return GetEnumerator();
        }
    }
}
